/*
 * Copyright (c) 2005 Silicon Graphics, Inc.
 * All rights reserved.
 *
 *   Michael A. Raymond <mraymond@sgi.com>
 */

/*
 *  This file is part of the ULI sample applications.
 *
 *  fork_test is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  fork_test is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with uli_sample; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <uli.h>

/*
 * This program tests the ULI safety mechanisms by having the registering
 * process go away while the ULI is forced to stay around.  It should
 * exit cleanly.
 */

#define ULI_CPU 0
void * ULIreg;
volatile int uli_data = 0;

/*
 * Modify a global variable so the main program can see that we ran.
 */
static void
uli_handler(void * arg_data)
{
	int * data = (int*)arg_data;

	(*data)++;

	return;
}

int
main(int argc, char * argv[])
{
	int ret;
	int line;

	/* Make sure an IRQ line was requested */
	if (argc < 2) {
		fprintf(stderr, "Must supply an IRQ line to use\n");
		return 1;
	}

	line = atoi(argv[1]);

	/* Lock down all the memory */
	if (mlockall(MCL_CURRENT | MCL_FUTURE) < 0) {
		perror("mlockall");
		return 1;
	}

	/* Register the handler */
	printf("Calling ULI_register\n");
	ULIreg = ULI_register_irq(ULI_CPU, uli_handler, (void*)&uli_data, 0,
				     NULL, 0, line);

	if (!ULIreg) {
		fprintf(stderr, "Problem with ULI_register_irq");
		return 1;
	}
	printf("Did registration\n");

	/*
	 * Fork off a process to hold the ULI file descriptor open.  Have
	 * the process with the registering address space exit.  This
	 * ensures the the ULI handler will attempt to run within an
	 * exiting or exited process.
	 */
	ret = fork();
	if (ret < 0) {
		perror("fork");
		exit(1);
	} else if (ret == 0) {
		printf("Child created and sleeping\n");
		sleep(10);
		printf("Child slept and exiting\n");
		return 0;
	} else {
		printf("Parent forked and exiting\n");
		return 0;
	}

	return 0;
}
